/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const CLASS_ID = Components.ID("594fe6be-0f6b-4aae-a19d-570543633cb2");
const CLASS_NAME = "ThumbsDown Script Manager";
const CONTRACT_ID = "@thumbsdown.mozdev.org/script-manager;1";
const CATEGORY_ENTRY = "thumbsdown";

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

const TYPE_DIRECT = 0;
const TYPE_REGEXP = 1;
const TYPE_CUSTOM = 2;


function ScriptManager()
{
    this.wrappedJSObject = this;
    this._scripts = null;
    this.prevMatch = null;
    
    var loader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
    loader.loadSubScript("chrome://thumbsdown/content/sql.js");
    loader.loadSubScript("chrome://thumbsdown/content/sqlConfig.js");
    
    this.sqlExec = new thumbsdown_SqlExecutor(THUMBSDOWN_SQLITE_FILENAME, THUMBSDOWN_SQL_EXECUTOR_CONFIG, THUMBSDOWN_SQL_EXECUTOR_INIT_CONFIG);
}

ScriptManager.prototype =
{
    get scripts()
    {
        if (this._scripts == null)
            this._scripts = this._loadScripts();
        return this._scripts;
    },
    
    reset: function()
    {
        this._scripts = null;
        this.prevMatch = null;
    },
    
    addScript: function(script)
    {
        var id = this.sqlExec.execute("insertScript", script);
        this.reset();
        return id;
    },
    
    getScript: function(id)
    {
        return this.sqlExec.fetchOne("getScript", id);
    },
    
    setScript: function(script)
    {
        this.sqlExec.execute("updateScript", script);
        this.reset();
    },
    
    removeScript: function(id)
    {
        this.sqlExec.execute("removeScript", id);
        this.reset();
    },
    
    lookup: function(linkUrl, thumbnailUrl)
    {
        var script = this._lookup(linkUrl, thumbnailUrl, this.prevMatch);
        if (script)
            this.prevMatch = script;
        return script;
    },
    
    _lookup: function(linkUrl, thumbnailUrl, script)
    {
        if (script)
            if (this._match(script, linkUrl, thumbnailUrl))
                return script;
        for each (script in this.scripts)
            if (this._match(script, linkUrl, thumbnailUrl))
                return script;
    },
    
    _match: function(script, linkUrl, thumbnailUrl)
    {
        if (! script.regexp.test(linkUrl))
            return false;
        switch (script.type)
        {
            case 0: return true;
            case 1:
                if (script.isSourceThumbnail)
                    return (thumbnailUrl && script.pattern.test(thumbnailUrl));
                else
                    return script.pattern.test(linkUrl);
            case 2: return true;
            default:
                throw new Error("unreachable code");
        }
    },
    
    _loadScripts: function()
    {
        var scripts1 = this.sqlExec.fetchAll("getScripts");
        var scripts2 = [];
        for each (script in scripts1)
        {
            if ((0 <= script.type) && (script.type <= 2))
            {
                this._initScript(script);
                scripts2.push(script);
            }
        }
        return scripts2;
    },
    
    _initScript: function(script)
    {
        script.regexp = new RegExp("^" + script.regexp + "$");
        if (script.type == TYPE_REGEXP)
        {
            script.isSourceThumbnail = (script.source == 0);
            script.isSourceLink = (script.source == 1);
            script.pattern = new RegExp("^" + script.pattern + "$");
        }
    },
    
    // nsISupports
    QueryInterface: function(iid)
    {
        if (iid.equals(Ci.nsISupports) ||
            iid.equals(Ci.nsISupportsWeakReference))
            return this;
        throw Cr.NS_ERROR_NO_INTERFACE;
    }
}


// nsIFactory
var ScriptManagerFactory =
{
    instance: null,
    
    createInstance: function(outer, iid)
    {
        if (outer != null)
            throw Cr.NS_ERROR_NO_AGGREGATION;
        if (this.instance == null)
            this.instance = new ScriptManager();
        return this.instance.QueryInterface(iid);
    }
};

// nsIModule
var ScriptManagerModule =
{
    registerSelf: function(compMgr, fileSpec, location, type)
    {
        // register
        compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, fileSpec, location, type);
        // add start-up observer
        //var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
        //catMgr.addCategoryEntry("xpcom-startup", CATEGORY_ENTRY, CONTRACT_ID, true, true);
    },
    
    unregisterSelf: function(compMgr, fileSpec, location)
    {
        // remove start-up observer
        //var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
        //catMgr.deleteCategoryEntry("xpcom-startup", CATEGORY_ENTRY, true);
        // unregister
        compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(CLASS_ID, location);
    },
    
    getClassObject: function(compMgr, cid, iid)
    {
        if (! cid.equals(CLASS_ID))
            throw Cr.NS_ERROR_NO_INTERFACE;
        if (! iid.equals(Ci.nsIFactory))
            throw Cr.NS_ERROR_NOT_IMPLEMENTED;
        return ScriptManagerFactory;
    },
    
    canUnload: function(compMgr)
    {
        return true;
    }
};

function NSGetModule(compMgr, fileSpec)
{
    return ScriptManagerModule;
}
